#include "TestLib.h"
#include "Main-App.h"

// 0 - 31 	:	Registertests
// 32 		:	Push-Pop-Return-Jump-Test
// 33 - 38	:	Arithmetische Tests
// 39 - 45	:	Tests der logischen Bitoperationen
// 46 - 48	:	Tests der logischen Operationen
// 49 - 52 	:	Tests der Transfer-Befehle
// 53		: 	RAM-Test
// 54		:	Port-Test
// 96 - 98	:	Timer-Tests
// 99		:	Watchdog-Test

int iTestAnzahl = 53;				// Die Anzahl der zyklisch zu durchlaufenden Tests
int iTestStat = 0;					// Der aktuell ausgewhlte selbsttest

int iWDSpeicher = 0x02;				// Speicherzelle 3 deklarieren

int iRamTemp = RamA;			// Variable beinhaltet die erste zu testende Ramzelle
int iRamAnz = 10;			// Anzahl der zu testenden Ramzellen pro Zyklus

void selectTest(){
	startTest(iTestStat);
}

// fhrt den Gewhlten Test aus!		
void startTest(int iTestNr){
	iTestStat = iTestNr;
	switch(iTestNr){
	
		// Registertests
		case 0: TEST_R0(); break;
		case 1: TEST_R1(); break;
		case 2: TEST_R2(); break;
		case 3: TEST_R3(); break;
		case 4: TEST_R4(); break;
		case 5: TEST_R5(); break;
		case 6: TEST_R6(); break;
		case 7: TEST_R7(); break;
		case 8: TEST_R8(); break;
		case 9: TEST_R9(); break;
		case 10: TEST_R10(); break;
		case 11: TEST_R11(); break;
		case 12: TEST_R12(); break;
		case 13: TEST_R13(); break;
		case 14: TEST_R14(); break;
		case 15: TEST_R15(); break;
		case 16: TEST_R16(); break;
		case 17: TEST_R17(); break;
		case 18: TEST_R18(); break;
		case 19: TEST_R19(); break;
		case 20: TEST_R20(); break;
		case 21: TEST_R21(); break;
		case 22: TEST_R22(); break;
		case 23: TEST_R23(); break;
		case 24: TEST_R24(); break;
		case 25: TEST_R25(); break;
		case 26: TEST_R26(); break;
		case 27: TEST_R27(); break;
		case 28: TEST_R28(); break;
		case 29: TEST_R29(); break;
		case 30: TEST_R30(); break;
		case 31: TEST_R31(); break;
		
		// Push-Pop-Ret-Test
		case 53: PPRJ_TEST(); break;
		
		// Arithmetische Tests
		case 33: ADD_TEST(); break;
		case 34: ADDC_TEST(); break;
		case 35: SUB_TEST(); break;
		case 36: INC_TEST(); break;
		case 37: DEC_TEST(); break;
		case 38: MUL_TEST(); break;

		// Tests der logischen Bitoperationen
		case 39: CLR_TEST(); break;
		case 40: COM_TEST(); break;
		case 41: LSL_TEST(); break;
		case 42: LSR_TEST(); break;
		case 43: ROL_TEST(); break;
		case 44: ROR_TEST(); break;
		case 45: SWAP_TEST(); break;

		// Tests der logischen Operationen
		case 46: AND_TEST(); break;
		case 47: OR_TEST(); break;
		case 48: EOR_TEST(); break;

		// Tests der Transfer-Befehle
		case 49: mov_test(); break;
		case 50: movw_test(); break;
		case 51: ldi_test(); break;
		case 52: ld_test(); break;
		
		// Test des Rams
		case 32: Ram_Test(); break;
		
		// Port-Test
		case 54: PortTest(); break;
		
		// Tests der Timer
		case 96: Timer0Test(); break;
		case 97: Timer1Test(); break;
		case 98: Timer2Test(); break; 
		
		// Watchdog-Test
		case 99: WDTest(); break;
		
		// Sonstiges
		default: stdError(); break;
	}
}

// fhrt alle vorhanden SelfTests aus
// Bsp. fr Startberprfung!
void initTest(){
	while(iTestStat <= iTestAnzahl){
		selectTest();
		iTestStat++;
	}
	iTestStat = 0;
}


// Methode um die Ports auf korrekte Funktion zu berprfen
void PortTest(){
	uint8_t iPruefVar = 0;								// Prf-Variable 
	int iDDRB = DDRB;								// Wert der alten Port-Zustnde speichern
	int iDDRD = DDRD;
	int iPortB = PORTB;
	int iPortD = PORTD;
	
	DDRB = 0x00;									// PortB als Eingang 
	DDRD = 0xFF;									// PortD als Ausgang
	
	// Prf-0-Test
	iPruefVar = ~0x01;								// Prf-0 vorladen
	PORTD = iPruefVar;
	for(int i=0;i<=7;i++){	
		if(PINB != iPruefVar)						// Vergleichen von PortB und pruefVar und falls ntig,
			testError();							// in Fehlermeldung springen
		iPruefVar = (iPruefVar<<1) | 0x01;			// 0 schieben
		PORTD = iPruefVar;
	}
	
	// Prf-1-Test
	/*
	iPruefVar = 0x01;								// Prf-1 vorladen
	PORTD = iPruefVar;
	for(int i=0;i<=7;i++){
		if(PINB != iPruefVar)						// Vergleichen von PortB und pruefVar und falls ntig,
			testError();							// in Fehlermeldung springen
		iPruefVar = (iPruefVar << 1);				// 1 schieben	
		PORTD = iPruefVar;							
	}
	*/
	
	// alte Zustnde zurck setzen
	DDRB = iDDRB;	
	PORTB = iPortB;
		
	DDRD = iDDRD;
	PORTD = iPortD;
	
}

// Routine fr den einfacheren RAM-Test
void Ram_Test(){
	if((iRamTemp + iRamAnz) > RamE){				// Prfen ob Prfrahmen ber Ramende hinaus
		iRamAnz = (iRamTemp + iRamAnz) - RamE;
		Ram_Check(iRamTemp, RamE);					// Wenn ja, bis zum Ende prfen und 
		iRamTemp = RamA;							// Pointer auf Anfang setzten
	}
	
	// Ram_Test durchfhren.
	Ram_Check(iRamTemp, (iRamTemp + iRamAnz));
		
	// Ramzelle auf nchste Zelle setzen (falls Ramende, dann auf Ramanfang)
	iRamTemp = iRamTemp + 1;
	if(iRamTemp > RamE)
		iRamTemp = RamA;
	
}

// eigentlicher Ram_Test
void Ram_Check(int iAadress, int iEadress){
	unsigned char *iAdress;
	iAdress = (unsigned char*)iAadress;			// Adresse bernehmen
	int iTemp; 										// Variable fr Speicherzelleninhalt
	
	// solange nicht alle Zellen getestet wurden
	while(iAdress < iEadress){
		iTemp = *iAdress;							// Inhalt speichern
		*iAdress = 0x55;							// Testwerte schreiben und vergleichen
		if(!*iAdress == 0x55)
			testError();
		*iAdress = 0xAA;
		if(!*iAdress == 0xAA)
			testError();
		*iAdress = iTemp;							// Inhalt zurck schreiben
		iAdress++;									// Nchste Zelle
	}
}

// Watchdog-Testroutine
void WDTest(){
	eepromWriteTo(iWDSpeicher, 1);				// Watchdog-Test-Status im EEPROM festhalten
	
	wdt_enable(WDTO_15MS);							// Watchdog aufziehen
	_delay_loop_2(5*DelayMS);						// Warten
	_delay_loop_2(5*DelayMS);
	_delay_loop_2(5*DelayMS);					
	_delay_loop_2(5*DelayMS);						// Wenn er hier nicht zuschnappt ist er wohl defekt

	testError();									// in Fehlermeldung springen
}

// Timer0-Testroutine
void Timer0Test()
{
	int iZeit[5], i;
	
	OCR0A = 10*DELAY;								// Ldt das Vergleichsregister (15mS)
	TCNT0 = 0;
	TCCR0A = (1<<WGM01);							// Initialisiert den Timer

	eepromWriteTo(iWDSpeicher, 2);				// Im EEPROM festhalten, dass Timer0Test luft.
	wdt_enable(WDTO_15MS);							// Aktiviert den Watchdog

	for (i = 0; i < 5; i++)
	{
		TCCR0A = (1<<CS00)|(1<<CS02);				// Timer starten
		_delay_loop_2(DelayMS);  					// Timer laufen lassen 	
		TCCR0A &= ~((1<<CS00)|(1<<CS02));			// Timer stoppen
		iZeit[i] = TCNT0;
		if (iZeit[i] != ((i+1) * DELAY))	 		// Vergleichen mit Erwartungshaltung
		  testError();
	}
	
	wdt_disable();									// Deaktiviert den Watchdog
}

// Timer1-Testroutine
void Timer1Test()
{
	unsigned int iZeit[10], i;
	
	OCR1A = 10*DELAY;								// Ldt das Vergleichsregister (15mS)
	TCNT1 = 0;
	TCCR1A = (1<<WGM11);							// Initialisiert den Timer

	eepromWriteTo(iWDSpeicher, 3);				// Im EEPROM festhalten, dass Timer0Test luft.
	wdt_enable(WDTO_15MS);							// Aktiviert den Watchdog

	for (i = 0; i < 5; i++)
	{
		TCCR1B = (1<<CS10)|(1<<CS12);				// Timer starten
		_delay_loop_2(DelayMS);  					// Timer laufen lassen 	
		TCCR1B &= ~((1<<CS10)|(1<<CS12));			// Timer stoppen
		iZeit[i] = TCNT1;
		if (iZeit[i] != ((i+1) * DELAY))	 		// Vergleichen mit Erwartungshaltung
		  testError();
	}
	
	wdt_disable();									// Deaktiviert den Watchdog
}

// Timer2-Testroutine
void Timer2Test()
{
	int iZeit[5], i;
	
	OCR2A = 10*DELAY;								// Ldt das Vergleichsregister (15mS)
	TCNT2 = 0;
	TCCR2A = (1<<WGM21);							// Initialisiert den Timer

	eepromWriteTo(iWDSpeicher, 4);					// Im EEPROM festhalten, dass Timer0Test luft.
	wdt_enable(WDTO_15MS);							// Aktiviert den Watchdog

	for (i = 0; i < 5; i++)
	{
		TCCR2A = (1<<CS20)|(1<<CS21)|(1<<CS22);	// Timer starten
		_delay_loop_2(DelayMS);  					// Timer laufen lassen 	
		TCCR2A &= ~((1<<CS20)|(1<<CS21)|(1<<CS22));			// Timer stoppen
		iZeit[i] = TCNT2;
		if (iZeit[i] != ((i+1) * DELAY))	 		// Vergleichen mit Erwartungshaltung
		  testError();
	}

	wdt_disable();									// Deaktiviert den Watchdog
}


// Routine um in den EEPROM zu schreiben
void eepromWriteTo(int iSpeicher, int iInfo){
		eeprom_write_byte((uint8_t*)iSpeicher, iInfo);
}
	
